@@ -6,7 +6,7 @@ use strict;
 use Carp qw( croak );
 
 use vars qw($VERSION);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.354'; # NOTE - Should be #.### (three decimal places)
 
 sub import {
   my $self = shift;
@@ -220,11 +220,11 @@ L<POE::Component::IKC|POE::Component::IKC> and L<POE::TIKC|POE::TIKC>.
 
 =head2 Layer 0: POE's Internals
 
-POE's layered architecture continues below the surface.  POE's guts are
-broken into specific L<POE::Loop|POE::Loop> classes for each event loop it supports. 
-Internals are divided up by type, giving L<POE::Resource|POE::Resource> classes for
-Aliases, Controls, Events, Extrefs, FileHandles, SIDs, Sessions, Signals,
-and Statistics.
+POE's layered architecture continues below the surface.  POE's guts
+are broken into specific L<POE::Loop|POE::Loop> classes for each event
+loop it supports.  Internals are divided up by type, giving
+L<POE::Resource|POE::Resource> classes for Aliases, Controls, Events,
+Extrefs, FileHandles, SIDs, Sessions and Signals.
 
 POE::Kernel's APIs are extensible through POE::API mix-in classes.
 Some brave souls have even published new APIs on CPAN, such as
@@ -270,7 +270,7 @@ classes describe what's common among each class.  It's a good idea to at
 least skim the base class documentation since the subclasses tend not to
 rehash the common things.
 
-L<POE::Queue|POE::Queue>, L<POE::Resource|POE::Resource>, L<POE::Loop|POE::Loop>, and L<POE::API|POE::API> document the
+L<POE::Queue|POE::Queue>, L<POE::Resource|POE::Resource>, and L<POE::Loop|POE::Loop> document the
 concepts and sometimes the standard interfaces behind multiple
 subclasses.  You're encouraged to have a look.
 
@@ -374,10 +374,11 @@ seems to be testing there anymore.  Reports and patches are still
 welcome.
 
 Past versions of POE have been tested with Perl versions as far back
-as 5.004_03 and as recent as "blead", today's development build.  We
+as 5.6.2 and as recent as "blead", today's development build.  We
 can no longer guarantee each release will work everywhere, but we will
 be happy to work with you if you need special support for a really old
-system.
+system. You can always use older POE releases that works on your version,
+please check L<BackPAN|http://backpan.perl.org/authors/id/R/RC/RCAPUTO/>.
 
 POE's quality is due in large part to the fine work of Chris Williams
 and the other CPAN testers.  They have dedicated resources towards
@@ -405,6 +406,26 @@ through contributions of time, patches, and other resources.  Some of
 them are: Sean Puckett, Douglas Couch, Andrew Chen, Uhlarik Ondoej,
 Nick Williams, and Chris Williams (no relation).
 
+=head2 Linux/Unix Issues
+
+=head3 pty woes
+
+Some distributions chose to not completely setup the pseudo-tty
+support. This is needed for L<POE::Wheel::Run> to interact with the
+subprocess. If you see something like this while running C<make test>
+please look at your distribution's documentation on how to fix it. For
+example, on Debian-based systems the solution was to execute 
+"sudo apt-get install udev".
+
+  t/30_loops/io_poll/wheel_run.t ..................... 1/99
+  pty_allocate(nonfatal): posix_openpt(): No such file or directory at /usr/local/lib/perl/5.10.0/IO/Pty.pm line 24.
+  ...
+  Cannot open a pty at /home/apoc/poe/blib/lib/POE/Wheel/Run.pm line 251
+  Compilation failed in require at t/30_loops/io_poll/wheel_run.t line 24.
+  # Looks like you planned 99 tests but ran 5.
+  # Looks like your test exited with 22 just after 5.
+  t/30_loops/io_poll/wheel_run.t ..................... Dubious, test returned 22 (wstat 5632, 0x1600)
+
 =head2 Other Compatibility Issues
 
 None currently known.  See GETTING HELP below if you've run into
@@ -494,8 +515,7 @@ L<POE::Queue>, L<POE::Queue::Array>
 
 L<POE::Resource>, L<POE::Resource::Aliases>, L<POE::Resource::Events>,
 L<POE::Resource::Extrefs>, L<POE::Resource::FileHandles>,
-L<POE::Resource::Performance>, L<POE::Resource::SIDs>,
-L<POE::Resource::Sessions>, L<POE::Resource::Signals>
+L<POE::Resource::SIDs>, L<POE::Resource::Sessions>, L<POE::Resource::Signals>
 
 =head2 Helpers
 
@@ -617,6 +637,12 @@ Filters.  He greatly improved L<POE::Wheel::FollowTail|POE::Wheel::FollowTail>,
 contributions include the basic Block filter, as well as Stackable,
 RecordBlock, Grep and Map.
 
+=item Plixer International
+
+Plixer International is at L<http://plixer.com/>.  Their sponsorship
+has helped POE 1.300 and beyond be significantly more robust using
+iThreads, especially when using fork() in Windows.
+
 =item Robert Seifer
 
 Robert Seifer is <e-mail unknown>.  He rotates IRC nicknames
@@ -630,7 +656,7 @@ problem relating to anonymous subs, scope and @{} processing.
 
 =item Matt Sergeant
 
-Matt contributed L<POE::Kernel::Poll|POE::Kernel::Poll>, a more efficient way to watch
+Matt contributed C<POE::Kernel::Poll>, a more efficient way to watch
 multiple files than select().  It's since been moved to
 L<POE::Loop::IO_Poll|POE::Loop::IO_Poll>.
 
@@ -659,8 +685,8 @@ author of L<Sprocket>, a networking framework built on POE.
 Please contact the author if you've been forgotten and would like to
 be included here.
 
-TODO - This section has fallen into disrepair.  A POE historian needs
-to cull the CHANGES for the names of major contributors.
+Z<TODO - This section has fallen into disrepair.  A POE historian needs
+to cull the CHANGES for the names of major contributors.>
 
 =back
 
