@@ -4,7 +4,7 @@
 package POE::Resource::FileHandles;
 
 use vars qw($VERSION);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.354'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -51,8 +51,8 @@ sub FNO_MODE_RD      () { MODE_RD } # [ [ (fileno read mode structure)
 # --- BEGIN SUB STRUCT 1 ---        #
 sub FMO_REFCOUNT     () { 0      }  #     $fileno_total_use_count,
 sub FMO_ST_ACTUAL    () { 1      }  #     $requested_file_state (see HS_PAUSED)
-sub FMO_SESSIONS     () { 2      }  #     { $session_watching_this_handle =>
-                                    #       { $handle_watched_as =>
+sub FMO_SESSIONS     () { 2      }  #     { $session_id =>
+                                    #       { $file_descriptor =>
 # --- BEGIN SUB STRUCT 2 ---        #
 sub HSS_HANDLE       () { 0      }  #         [ $blessed_handle,
 sub HSS_SESSION      () { 1      }  #           $blessed_session,
@@ -81,9 +81,8 @@ sub HS_RUNNING   () { 0x02 }   # The file is running and can generate events.
 ### Handle to session.
 
 my %kr_ses_to_handle;
-
-                            #    { $session =>
-                            #      $handle =>
+                            #    { $session_id =>
+                            #      $fileno =>
 # --- BEGIN SUB STRUCT ---  #        [
 sub SH_HANDLE     () {  0 } #          $blessed_file_handle,
 sub SH_REFCOUNT   () {  1 } #          $total_reference_count,
@@ -96,6 +95,24 @@ sub SH_MODECOUNT  () {  2 } #          [ $read_reference_count,     (MODE_RD)
                             #      },
                             #    },
 
+sub _data_handle_relocate_kernel_id {
+  my ($self, $old_id, $new_id) = @_;
+
+  foreach my $fd_rec (values %kr_filenos) {
+    my $rd_rec = $fd_rec->[FNO_MODE_RD][FMO_SESSIONS];
+    $rd_rec->{$new_id} = delete $rd_rec->{$old_id} if exists $rd_rec->{$old_id};
+
+    my $wr_rec = $fd_rec->[FNO_MODE_WR][FMO_SESSIONS];
+    $wr_rec->{$new_id} = delete $wr_rec->{$old_id} if exists $wr_rec->{$old_id};
+
+    my $ex_rec = $fd_rec->[FNO_MODE_EX][FMO_SESSIONS];
+    $ex_rec->{$new_id} = delete $ex_rec->{$old_id} if exists $ex_rec->{$old_id};
+  }
+
+  $kr_ses_to_handle{$new_id} = delete $kr_ses_to_handle{$old_id}
+    if exists $kr_ses_to_handle{$old_id};
+}
+
 ### Begin-run initialization.
 
 sub _data_handle_initialize {
@@ -118,9 +135,9 @@ sub _data_handle_finalize {
       "!!!\tRead:\n",
       "!!!\t\trefcnt  = $rd->[FMO_REFCOUNT]\n",
     );
-    while (my ($ses, $ses_rec) = each(%{$rd->[FMO_SESSIONS]})) {
-      _warn "!!!\t\tsession = $ses\n";
-      while (my ($handle, $hnd_rec) = each(%{$ses_rec})) {
+    while (my ($sid, $ses_rec) = each(%{$rd->[FMO_SESSIONS]})) {
+      _warn "!!!\t\tsession $sid\n";
+      while (my ($fd, $hnd_rec) = each(%{$ses_rec})) {
         _warn(
           "!!!\t\t\thandle  = $hnd_rec->[HSS_HANDLE]\n",
           "!!!\t\t\tsession = $hnd_rec->[HSS_SESSION]\n",
@@ -134,9 +151,9 @@ sub _data_handle_finalize {
       "!!!\tWrite:\n",
       "!!!\t\trefcnt  = $wr->[FMO_REFCOUNT]\n",
     );
-    while (my ($ses, $ses_rec) = each(%{$wr->[FMO_SESSIONS]})) {
-      _warn "!!!\t\tsession = $ses\n";
-      while (my ($handle, $hnd_rec) = each(%{$ses_rec})) {
+    while (my ($sid, $ses_rec) = each(%{$wr->[FMO_SESSIONS]})) {
+      _warn "!!!\t\tsession = $sid\n";
+      while (my ($fd, $hnd_rec) = each(%{$ses_rec})) {
         _warn(
           "!!!\t\t\thandle  = $hnd_rec->[HSS_HANDLE]\n",
           "!!!\t\t\tsession = $hnd_rec->[HSS_SESSION]\n",
@@ -150,9 +167,9 @@ sub _data_handle_finalize {
       "!!!\tException:\n",
       "!!!\t\trefcnt  = $ex->[FMO_REFCOUNT]\n",
     );
-    while (my ($ses, $ses_rec) = each(%{$ex->[FMO_SESSIONS]})) {
-      _warn "!!!\t\tsession = $ses\n";
-      while (my ($handle, $hnd_rec) = each(%{$ses_rec})) {
+    while (my ($sid, $ses_rec) = each(%{$ex->[FMO_SESSIONS]})) {
+      _warn "!!!\t\tsession = $sid\n";
+      while (my ($fd, $hnd_rec) = each(%{$ses_rec})) {
         _warn(
           "!!!\t\t\thandle  = $hnd_rec->[HSS_HANDLE]\n",
           "!!!\t\t\tsession = $hnd_rec->[HSS_SESSION]\n",
@@ -163,12 +180,12 @@ sub _data_handle_finalize {
     }
   }
 
-  while (my ($ses, $hnd_rec) = each(%kr_ses_to_handle)) {
+  while (my ($ses_id, $hnd_rec) = each(%kr_ses_to_handle)) {
     $finalized_ok = 0;
-    _warn "!!! Leaked handle in $ses\n";
-    while (my ($hnd, $rc) = each(%$hnd_rec)) {
+    _warn "!!! Leaked file descriptor in $ses_id\n";
+    while (my ($fd, $rc) = each(%$hnd_rec)) {
       _warn(
-        "!!!\tHandle: $hnd (tot refcnt=$rc->[SH_REFCOUNT])\n",
+        "!!!\tDescriptor: $fd (tot refcnt=$rc->[SH_REFCOUNT])\n",
         "!!!\t\tRead      refcnt: $rc->[SH_MODECOUNT]->[MODE_RD]\n",
         "!!!\t\tWrite     refcnt: $rc->[SH_MODECOUNT]->[MODE_WR]\n",
         "!!!\t\tException refcnt: $rc->[SH_MODECOUNT]->[MODE_EX]\n",
@@ -283,12 +300,13 @@ sub _data_handle_add {
 
   # The session is already watching this fileno in this mode.
 
-  if ($kr_fno_rec->[FMO_SESSIONS]->{$session}) {
+  my $sid = $session->ID;
+  if ($kr_fno_rec->[FMO_SESSIONS]->{$sid}) {
 
     # The session is also watching it by the same handle.  Treat this
     # as a "resume" in this mode.
 
-    if (exists $kr_fno_rec->[FMO_SESSIONS]->{$session}->{$handle}) {
+    if (exists $kr_fno_rec->[FMO_SESSIONS]->{$sid}->{$fd}) {
       if (TRACE_FILES) {
         _warn("<fh> running $handle fileno($fd) mode($mode)");
       }
@@ -305,9 +323,9 @@ sub _data_handle_add {
     # have something registered for it here.
 
     else {
-      foreach my $watch_session (keys %{$kr_fno_rec->[FMO_SESSIONS]}) {
+      foreach my $watch_sid (keys %{$kr_fno_rec->[FMO_SESSIONS]}) {
         foreach my $hdl_rec (
-          values %{$kr_fno_rec->[FMO_SESSIONS]->{$watch_session}}
+          values %{$kr_fno_rec->[FMO_SESSIONS]->{$watch_sid}}
         ) {
           my $other_handle = $hdl_rec->[HSS_HANDLE];
 
@@ -322,7 +340,7 @@ sub _data_handle_add {
             $why = "open with different file descriptor";
           }
 
-          if ($session eq $watch_session) {
+          if ($sid eq $watch_sid) {
             _die(
               "A session was caught watching two different file handles that\n",
               "reference the same file descriptor in the same mode ($mode).\n",
@@ -331,7 +349,8 @@ sub _data_handle_add {
               "first unregistering it from POE.\n",
               "\n",
               "Some possibly helpful information:\n",
-              "  Session    : ", $self->_data_alias_loggable($session), "\n",
+              "  Session    : ",
+              $self->_data_alias_loggable($sid), "\n",
               "  Old handle : $other_handle (currently $why)\n",
               "  New handle : $handle\n",
               "\n",
@@ -347,9 +366,10 @@ sub _data_handle_add {
               "\n",
               "Some possibly helpful information:\n",
               "  Old session: ",
-              $self->_data_alias_loggable($hdl_rec->[HSS_SESSION]), "\n",
+              $self->_data_alias_loggable($hdl_rec->[HSS_SESSION]->ID), "\n",
               "  Old handle : $other_handle (currently $why)\n",
-              "  New session: ", $self->_data_alias_loggable($session), "\n",
+              "  New session: ",
+              $self->_data_alias_loggable($sid), "\n",
               "  New handle : $handle\n",
               "\n",
               "Please correct the program and try again.\n",
@@ -365,7 +385,7 @@ sub _data_handle_add {
   # the session/handle pair.
 
   else {
-    $kr_fno_rec->[FMO_SESSIONS]->{$session}->{$handle} = [
+    $kr_fno_rec->[FMO_SESSIONS]->{$sid}->{$fd} = [
       $handle,   # HSS_HANDLE
       $session,  # HSS_SESSION
       $event,    # HSS_STATE
@@ -388,8 +408,8 @@ sub _data_handle_add {
   # If the session hasn't already been watching the filehandle, then
   # register the filehandle in the session's structure.
 
-  unless (exists $kr_ses_to_handle{$session}->{$handle}) {
-    $kr_ses_to_handle{$session}->{$handle} = [
+  unless (exists $kr_ses_to_handle{$sid}->{$fd}) {
+    $kr_ses_to_handle{$sid}->{$fd} = [
       $handle,  # SH_HANDLE
       0,        # SH_REFCOUNT
       [ 0,      # SH_MODECOUNT / MODE_RD
@@ -397,13 +417,13 @@ sub _data_handle_add {
         0       # SH_MODECOUNT / MODE_EX
       ]
     ];
-    $self->_data_ses_refcount_inc($session);
+    $self->_data_ses_refcount_inc($sid);
   }
 
   # Modify the session's handle structure's reference counts, so the
   # session knows it has a reason to live.
 
-  my $ss_handle = $kr_ses_to_handle{$session}->{$handle};
+  my $ss_handle = $kr_ses_to_handle{$sid}->{$fd};
   unless ($ss_handle->[SH_MODECOUNT]->[$mode]) {
     $ss_handle->[SH_MODECOUNT]->[$mode]++;
     $ss_handle->[SH_REFCOUNT]++;
@@ -418,52 +438,33 @@ sub _data_handle_condition {
     # tied handle that doesn't support binmode.
     eval { binmode *$handle };
 
-    # Turn off blocking unless it's tied or a plain file.
-    unless (tied *$handle or -f $handle) {
+    # Turn off blocking on the handle.  Requires a sufficiently
+    # advanced Perl as not to be broken.  Otherwise we must skip tied
+    # filehandles or plain files.
+    #
+    # Perl-5.6.2 and older seem to hate tied FHs or plain files, so we
+    # be careful!
+    #
+    # ok 115 - regular file: handle removed fully
+    # Bad filehandle: GEN11
+    #   at /home/cpan/poe/blib/lib/POE/Resource/FileHandles.pm line 442.
+    # Compilation failed in require
+    #   at t/20_resources/10_perl/filehandles.t line 9.
 
-      unless (RUNNING_IN_HELL) {
-        if ($] >= 5.008) {
-          $handle->blocking(0);
-        }
-        else {
-          # Long, drawn out, POSIX way.
-          my $flags = fcntl($handle, F_GETFL, 0)
-            or _trap "fcntl($handle, F_GETFL, 0) fails: $!\n";
-          until (fcntl($handle, F_SETFL, $flags | O_NONBLOCK)) {
-            _trap(
-              "fcntl($handle [" . fileno($handle) . "], F_SETFL [" .
-              F_SETFL . "], $flags | O_NONBLOCK [" . O_NONBLOCK .
-              "]) fails: $!"
-            ) unless $! == EAGAIN or $! == EWOULDBLOCK;
-          }
-        }
-      }
-      else {
-        # Do it the Win32 way.
-        my $set_it = "1";
-
-        # 126 is FIONBIO (some docs say 0x7F << 16)
-        ioctl(
-          $handle,
-          0x80000000 | (4 << 16) | (ord('f') << 8) | 126,
-          \$set_it
-        ) or _trap(
-          "ioctl($handle, FIONBIO, $set_it) fails: errno " . ($!+0) . " = $!\n"
-        );
-      }
-    }
+    IO::Handle::blocking($handle, 0) if (
+      $] >= 5.008001 or not (tied *$handle or -f $handle)
+    );
 
     # Turn off buffering.
+    # you may be tempted to use $handle->autoflush(1) BUT DON'T DO THAT! ( things blow up )
     CORE::select((CORE::select($handle), $| = 1)[0]);
 }
 
-
-
 ### Remove a select from the kernel, and possibly trigger the
 ### session's destruction.
 
 sub _data_handle_remove {
-  my ($self, $handle, $mode, $session) = @_;
+  my ($self, $handle, $mode, $sid) = @_;
   my $fd = fileno($handle);
 
   # Make sure the handle is deregistered with the kernel.
@@ -475,20 +476,19 @@ sub _data_handle_remove {
     # Make sure the handle was registered to the requested session.
 
     if (
-      exists($kr_fno_rec->[FMO_SESSIONS]->{$session}) and
-      exists($kr_fno_rec->[FMO_SESSIONS]->{$session}->{$handle})
+      exists($kr_fno_rec->[FMO_SESSIONS]->{$sid}) and
+      exists($kr_fno_rec->[FMO_SESSIONS]->{$sid}->{$fd})
     ) {
 
       TRACE_FILES and
         _warn(
           "<fh> removing handle ($handle) fileno ($fd) mode ($mode) from " .
-          Carp::shortmess
+          $self->_data_alias_loggable($sid) . Carp::shortmess()
         );
 
       # Remove the handle from the kernel's session record.
 
-      my $handle_rec =
-        delete $kr_fno_rec->[FMO_SESSIONS]->{$session}->{$handle};
+      my $handle_rec = delete $kr_fno_rec->[FMO_SESSIONS]->{$sid}->{$fd};
 
       my $kill_session = $handle_rec->[HSS_SESSION];
       my $kill_event   = $handle_rec->[HSS_STATE];
@@ -505,7 +505,10 @@ sub _data_handle_remove {
 
       foreach ($kr_queue->remove_items($my_select)) {
         my ($time, $id, $event) = @$_;
-        $self->_data_ev_refcount_dec( @$event[EV_SESSION, EV_SOURCE] );
+        $self->_data_ev_refcount_dec(
+          $event->[EV_SOURCE]->ID(),
+          $event->[EV_SESSION]->ID(),
+        );
 
         TRACE_EVENTS and _warn(
           "<ev> removing select event $id ``$event->[EV_NAME]''" .
@@ -531,8 +534,8 @@ sub _data_handle_remove {
 
         # The session is not watching handles anymore.  Remove the
         # session entirely the fileno structure.
-        delete $kr_fno_rec->[FMO_SESSIONS]->{$session}
-          unless keys %{$kr_fno_rec->[FMO_SESSIONS]->{$session}};
+        delete $kr_fno_rec->[FMO_SESSIONS]->{$sid}
+          unless keys %{$kr_fno_rec->[FMO_SESSIONS]->{$sid}};
       }
 
       # Decrement the kernel record's handle reference count.  If the
@@ -562,7 +565,9 @@ sub _data_handle_remove {
   }
   elsif (TRACE_FILES) {
     _warn(
-      "<fh> handle ($handle) fileno ($fd) is not registered with POE::Kernel"
+      "<fh> handle ($handle) fileno ($fd) is not registered with POE::Kernel" .
+      Carp::shortmess()
+
     );
   }
 
@@ -570,13 +575,13 @@ sub _data_handle_remove {
   # a session to remove it from.  TODO Key it on fileno?
 
   if (
-    exists($kr_ses_to_handle{$session}) and
-    exists($kr_ses_to_handle{$session}->{$handle})
+    exists($kr_ses_to_handle{$sid}) and
+    exists($kr_ses_to_handle{$sid}->{$fd})
   ) {
 
     # Remove it from the session's read, write or expedite mode.
 
-    my $ss_handle = $kr_ses_to_handle{$session}->{$handle};
+    my $ss_handle = $kr_ses_to_handle{$sid}->{$fd};
     if ($ss_handle->[SH_MODECOUNT]->[$mode]) {
 
       # Hmm... what is this?  Was POE going to support multiple selects?
@@ -593,16 +598,16 @@ sub _data_handle_remove {
       }
 
       unless ($ss_handle->[SH_REFCOUNT]) {
-        delete $kr_ses_to_handle{$session}->{$handle};
-        $self->_data_ses_refcount_dec($session);
-        delete $kr_ses_to_handle{$session}
-          unless keys %{$kr_ses_to_handle{$session}};
+        delete $kr_ses_to_handle{$sid}->{$fd};
+        $self->_data_ses_refcount_dec($sid);
+        delete $kr_ses_to_handle{$sid}
+          unless keys %{$kr_ses_to_handle{$sid}};
       }
     }
     elsif (TRACE_FILES) {
       _warn(
         "<fh> handle ($handle) fileno ($fd) is not registered with",
-        $self->_data_alias_loggable($session)
+        $self->_data_alias_loggable($sid)
       );
     }
   }
@@ -657,27 +662,24 @@ sub _data_handle_count {
 ### Return the number of active handles for a single session.
 
 sub _data_handle_count_ses {
-  my ($self, $session) = @_;
-  return 0 unless exists $kr_ses_to_handle{$session};
-  return scalar keys %{$kr_ses_to_handle{$session}};
+  my ($self, $sid) = @_;
+  return 0 unless exists $kr_ses_to_handle{$sid};
+  return scalar keys %{$kr_ses_to_handle{$sid}};
 }
 
 ### Clear all the handles owned by a session.
 
 sub _data_handle_clear_session {
-  my ($self, $session) = @_;
-  return unless exists $kr_ses_to_handle{$session}; # avoid autoviv
-  my @handles = values %{$kr_ses_to_handle{$session}};
-  foreach (@handles) {
+  my ($self, $sid) = @_;
+
+  return unless exists $kr_ses_to_handle{$sid}; # avoid autoviv
+  foreach (values %{$kr_ses_to_handle{$sid}}) {
     my $handle = $_->[SH_HANDLE];
     my $refcount = $_->[SH_MODECOUNT];
 
-    $self->_data_handle_remove($handle, MODE_RD, $session)
-      if $refcount->[MODE_RD];
-    $self->_data_handle_remove($handle, MODE_WR, $session)
-      if $refcount->[MODE_WR];
-    $self->_data_handle_remove($handle, MODE_EX, $session)
-      if $refcount->[MODE_EX];
+    $self->_data_handle_remove($handle, MODE_RD, $sid) if $refcount->[MODE_RD];
+    $self->_data_handle_remove($handle, MODE_WR, $sid) if $refcount->[MODE_WR];
+    $self->_data_handle_remove($handle, MODE_EX, $sid) if $refcount->[MODE_EX];
   }
 }
 
