@@ -4,7 +4,7 @@
 package POE::Resource::SIDs;
 
 use vars qw($VERSION);
-$VERSION = '1.289'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.354'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -19,18 +19,19 @@ my %kr_session_ids;
 #    ...,
 #  );
 
-my %kr_session_to_id;
-#  ( $session_ref => $session_id,
-#    ...,
-#  );
-
-my $kr_sid_seq = 1;
+my $kr_sid_seq = 0;
 
 sub _data_sid_initialize {
   $poe_kernel->[KR_SESSION_IDS] = \%kr_session_ids;
   $poe_kernel->[KR_SID_SEQ] = \$kr_sid_seq;
 }
 
+sub _data_sid_relocate_kernel_id {
+  my ($self, $old_id, $new_id) = @_;
+  $kr_session_ids{$new_id} = delete $kr_session_ids{$old_id}
+    if exists $kr_session_ids{$old_id};
+}
+
 ### End-run leak checking.
 
 sub _data_sid_finalize {
@@ -39,10 +40,6 @@ sub _data_sid_finalize {
     _warn "!!! Leaked session ID: $sid = $ses\n";
     $finalized_ok = 0;
   }
-  while (my ($ses, $sid) = each(%kr_session_to_id)) {
-    _warn "!!! Leak sid cross-reference: $ses = $sid\n";
-    $finalized_ok = 0;
-  }
   return $finalized_ok;
 }
 
@@ -59,18 +56,18 @@ sub _data_sid_allocate {
 sub _data_sid_set {
   my ($self, $sid, $session) = @_;
   $kr_session_ids{$sid} = $session;
-  $kr_session_to_id{$session} = $sid;
 }
 
 ### Clear a session ID.
 
 sub _data_sid_clear {
-  my ($self, $session) = @_;
-  my $sid = delete $kr_session_to_id{$session};
-  if (ASSERT_DATA) {
-    _trap("SID not defined") unless defined $sid;
-  }
-  delete $kr_session_ids{$sid};
+  my ($self, $sid) = @_;
+
+  return delete $kr_session_ids{$sid} unless ASSERT_DATA;
+
+  my $removed = delete $kr_session_ids{$sid};
+  _trap("unknown SID '$sid'") unless defined $removed;
+  $removed;
 }
 
 ### Resolve a session ID into its session.
